@@ -39,24 +39,24 @@
 #include "gnc-plugin.h"
 #include "gnc-plugin-page.h"
 #include "gnc-gobject-utils.h"
+#include "gnc-ui.h"
 
 /** The debugging module that this .o belongs to.  */
 static QofLogModule log_module = GNC_MOD_GUI;
-/** A pointer to the parent class of a plugin page. */
-static gpointer parent_class = NULL;
 
-static void gnc_plugin_page_class_init (GncPluginPageClass *klass);
-static void gnc_plugin_page_init       (GncPluginPage *plugin_page,
-		                        void *data);
+static void gnc_plugin_page_constructed (GObject *object);
 static void gnc_plugin_page_finalize   (GObject *object);
-static void gnc_plugin_page_set_property (GObject         *object,
-        guint            prop_id,
-        const GValue    *value,
-        GParamSpec      *pspec);
-static void gnc_plugin_page_get_property (GObject         *object,
-        guint            prop_id,
-        GValue          *value,
-        GParamSpec      *pspec);
+static void gnc_plugin_page_set_property (GObject      *object,
+                                          guint         prop_id,
+                                          const GValue *value,
+                                          GParamSpec   *pspec);
+static void gnc_plugin_page_get_property (GObject      *object,
+                                          guint         prop_id,
+                                          GValue       *value,
+                                          GParamSpec   *pspec);
+
+static void gnc_plugin_page_default_focus (GncPluginPage *plugin_page,
+                                           gboolean on_current_page);
 
 enum
 {
@@ -72,13 +72,10 @@ enum
     PROP_0,
     PROP_PAGE_NAME,
     PROP_PAGE_COLOR,
-    PROP_PAGE_URI,
     PROP_BOOK,
     PROP_STATUSBAR_TEXT,
     PROP_USE_NEW_WINDOW,
     PROP_UI_DESCRIPTION,
-    PROP_UI_MERGE,
-    PROP_ACTION_GROUP,
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -88,11 +85,14 @@ static guint signals[LAST_SIGNAL] = { 0 };
 typedef struct _GncPluginPagePrivate
 {
     /** The group of all actions provided by this plugin. */
-    GtkActionGroup *action_group;
-    GtkUIManager *ui_merge;
-    guint merge_id;
     char *ui_description;
 
+    GtkBuilder *builder;
+    GSimpleActionGroup *simple_action_group;
+    const gchar *simple_action_group_name;
+    const gchar *menu_qualifier;
+    const gchar *menu_popup_qualifier;
+
     GList *books;
 
     gboolean use_new_window;
@@ -102,13 +102,17 @@ typedef struct _GncPluginPagePrivate
     gchar *page_color;
     gchar *uri;
     gchar *statusbar_text;
+
+    gulong page_changed_id;
+    guint  focus_source_id;
+
 } GncPluginPagePrivate;
 
-GNC_DEFINE_TYPE_WITH_CODE(GncPluginPage, gnc_plugin_page, G_TYPE_OBJECT,
-		        G_ADD_PRIVATE(GncPluginPage))
+G_DEFINE_TYPE_WITH_CODE(GncPluginPage, gnc_plugin_page, G_TYPE_OBJECT,
+                G_ADD_PRIVATE(GncPluginPage))
 
 #define GNC_PLUGIN_PAGE_GET_PRIVATE(o)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNC_TYPE_PLUGIN_PAGE, GncPluginPagePrivate))
+     ((GncPluginPagePrivate*)gnc_plugin_page_get_instance_private((GncPluginPage*)o))
 
 /*  Create the display widget that corresponds to this plugin.  This
  *  function will be called by the main/embedded window manipulation
@@ -121,9 +125,9 @@ gnc_plugin_page_create_widget (GncPluginPage *plugin_page)
     GncPluginPageClass *klass;
     GtkWidget *widget;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page), NULL);
 
-    klass = GNC_PLUGIN_PAGE_GET_CLASS (plugin_page);
+    klass = GNC_PLUGIN_PAGE_GET_CLASS(plugin_page);
     g_return_val_if_fail (klass != NULL, NULL);
     g_return_val_if_fail (klass->create_widget != NULL, NULL);
 
@@ -136,7 +140,7 @@ gnc_plugin_page_create_widget (GncPluginPage *plugin_page)
      * main notebook for the window.
      */
     if (klass->destroy_widget)
-        g_object_ref(widget);
+        g_object_ref (widget);
 
     return widget;
 }
@@ -150,9 +154,9 @@ gnc_plugin_page_destroy_widget (GncPluginPage *plugin_page)
 {
     GncPluginPageClass *klass;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
 
-    klass = GNC_PLUGIN_PAGE_GET_CLASS (plugin_page);
+    klass = GNC_PLUGIN_PAGE_GET_CLASS(plugin_page);
     g_return_if_fail (klass != NULL);
     g_return_if_fail (klass->destroy_widget != NULL);
 
@@ -165,19 +169,15 @@ void
 gnc_plugin_page_show_summarybar (GncPluginPage *page,
                                  gboolean visible)
 {
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     if (!page->summarybar)
         return;
 
     if (visible)
-    {
-        gtk_widget_show(page->summarybar);
-    }
+        gtk_widget_show (page->summarybar);
     else
-    {
-        gtk_widget_hide(page->summarybar);
-    }
+        gtk_widget_hide (page->summarybar);
 }
 
 
@@ -192,16 +192,16 @@ gnc_plugin_page_save_page (GncPluginPage *page,
 {
     GncPluginPageClass *klass;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
     g_return_if_fail (key_file != NULL);
     g_return_if_fail (group_name != NULL);
 
     ENTER(" ");
-    klass = GNC_PLUGIN_PAGE_GET_CLASS (page);
+    klass = GNC_PLUGIN_PAGE_GET_CLASS(page);
     g_return_if_fail (klass != NULL);
     g_return_if_fail (klass->save_page != NULL);
 
-    klass->save_page(page, key_file, group_name);
+    klass->save_page (page, key_file, group_name);
     LEAVE(" ");
 }
 
@@ -220,17 +220,17 @@ gnc_plugin_page_recreate_page(GtkWidget *window,
     GType type;
 
     ENTER("type %s, keyfile %p, group %s", page_type, key_file, page_group);
-    type = g_type_from_name(page_type);
+    type = g_type_from_name (page_type);
     if (type == 0)
     {
         LEAVE("Cannot find type named %s", page_type);
         return NULL;
     }
 
-    klass = g_type_class_ref(type);
+    klass = g_type_class_ref (type);
     if (klass == NULL)
     {
-        const gchar *type_name = g_type_name(type);
+        const gchar *type_name = g_type_name (type);
         LEAVE("Cannot create class %s(%s)", page_type, type_name ? type_name : "invalid type");
         return NULL;
     }
@@ -238,69 +238,58 @@ gnc_plugin_page_recreate_page(GtkWidget *window,
     if (!klass->recreate_page)
     {
         LEAVE("Class %shas no recreate function.", page_type);
-        g_type_class_unref(klass);
+        g_type_class_unref (klass);
         return NULL;
     }
 
     page = (klass->recreate_page)(window, key_file, page_group);
-    g_type_class_unref(klass);
+    g_type_class_unref (klass);
     LEAVE(" ");
     return page;
 }
 
 
-/*  Add the actions for a content page to the specified window. */
 void
-gnc_plugin_page_merge_actions (GncPluginPage *page,
-                               GtkUIManager *ui_merge)
+gnc_plugin_page_merge_actions (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
+    GError *error = NULL;
+    gchar *resource;
 
     g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    priv->ui_merge = ui_merge;
-    gtk_action_group_set_sensitive (priv->action_group, TRUE);
-    priv->merge_id = gnc_plugin_add_actions(priv->ui_merge,
-                                            priv->action_group,
-                                            priv->ui_description);
+
+    if (!priv->builder)
+        priv->builder = gtk_builder_new ();
+
+    resource = g_strconcat (GNUCASH_RESOURCE_PREFIX "/", priv->ui_description, NULL);
+
+    gtk_builder_set_translation_domain (priv->builder, PROJECT_NAME);
+
+    gtk_builder_add_from_resource (priv->builder, resource, &error);
+
+    if (error)
+    {
+        g_critical ("Failed to load ui resource %s, Error %s", resource, error->message);
+        g_error_free (error);
+    }
+    g_free (resource);
 }
 
 
-/*  Remove the actions for a content page from the specified window. */
-void
-gnc_plugin_page_unmerge_actions (GncPluginPage *page,
-                                 GtkUIManager *ui_merge)
-{
-    GncPluginPagePrivate *priv;
-
-    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
-    g_return_if_fail (priv->merge_id != 0);
-    g_return_if_fail (priv->action_group != NULL);
-
-    gtk_ui_manager_remove_ui(ui_merge, priv->merge_id);
-    gtk_action_group_set_sensitive (priv->action_group, FALSE);
-    gtk_ui_manager_remove_action_group(ui_merge, priv->action_group);
-
-    priv->ui_merge = NULL;
-    priv->merge_id = 0;
-}
-
-
-GtkAction *
+GAction *
 gnc_plugin_page_get_action (GncPluginPage *page, const gchar *name)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail(GNC_IS_PLUGIN_PAGE(page), NULL);
-    g_return_val_if_fail(name != NULL, NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
+    g_return_val_if_fail (name != NULL, NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    if (!priv->action_group)
+    if (!priv->simple_action_group)
         return NULL;
-    return gtk_action_group_get_action (priv->action_group, name);
+    return g_action_map_lookup_action (G_ACTION_MAP(priv->simple_action_group), name);
 }
 
 
@@ -310,9 +299,9 @@ gnc_plugin_page_get_plugin_name (GncPluginPage *plugin_page)
 {
     GncPluginPageClass *klass;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page), NULL);
 
-    klass = GNC_PLUGIN_PAGE_GET_CLASS (plugin_page);
+    klass = GNC_PLUGIN_PAGE_GET_CLASS(plugin_page);
     g_return_val_if_fail (klass != NULL, NULL);
 
     return (klass->plugin_name);
@@ -323,33 +312,33 @@ gnc_plugin_page_get_plugin_name (GncPluginPage *plugin_page)
 void
 gnc_plugin_page_inserted (GncPluginPage *plugin_page)
 {
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
 
-    g_signal_emit (G_OBJECT (plugin_page), signals[INSERTED], 0);
+    g_signal_emit (G_OBJECT(plugin_page), signals[INSERTED], 0);
 }
 
 void
 gnc_plugin_page_removed (GncPluginPage *plugin_page)
 {
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
 
-    g_signal_emit (G_OBJECT (plugin_page), signals[REMOVED], 0);
+    g_signal_emit (G_OBJECT(plugin_page), signals[REMOVED], 0);
 }
 
 void
 gnc_plugin_page_selected (GncPluginPage *plugin_page)
 {
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
 
-    g_signal_emit (G_OBJECT (plugin_page), signals[SELECTED], 0);
+    g_signal_emit (G_OBJECT(plugin_page), signals[SELECTED], 0);
 }
 
 void
 gnc_plugin_page_unselected (GncPluginPage *plugin_page)
 {
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (plugin_page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
 
-    g_signal_emit (G_OBJECT (plugin_page), signals[UNSELECTED], 0);
+    g_signal_emit (G_OBJECT(plugin_page), signals[UNSELECTED], 0);
 }
 
 /** Initialize the class for a new generic plugin page.  This will set
@@ -362,15 +351,16 @@ gnc_plugin_page_unselected (GncPluginPage *plugin_page)
 static void
 gnc_plugin_page_class_init (GncPluginPageClass *klass)
 {
-    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-    parent_class = g_type_class_peek_parent (klass);
+    gobject_class->constructed = gnc_plugin_page_constructed;
     gobject_class->finalize = gnc_plugin_page_finalize;
     gobject_class->set_property = gnc_plugin_page_set_property;
     gobject_class->get_property = gnc_plugin_page_get_property;
 
     klass->tab_icon    = NULL;
     klass->plugin_name = NULL;
+    klass->focus_page = gnc_plugin_page_default_focus;
 
     g_object_class_install_property
     (gobject_class,
@@ -395,15 +385,6 @@ gnc_plugin_page_class_init (GncPluginPageClass *klass)
                           NULL,
                           G_PARAM_READWRITE));
 
-    g_object_class_install_property
-    (gobject_class,
-     PROP_PAGE_URI,
-     g_param_spec_string ("page-uri",
-                          "Page URI",
-                          "The uri for this page.",
-                          NULL,
-                          G_PARAM_READWRITE));
-
     g_object_class_install_property
     (gobject_class,
      PROP_STATUSBAR_TEXT,
@@ -435,28 +416,6 @@ gnc_plugin_page_class_init (GncPluginPageClass *klass)
                           NULL,
                           G_PARAM_READWRITE));
 
-    g_object_class_install_property
-    (gobject_class,
-     PROP_UI_MERGE,
-     g_param_spec_object ("ui-merge",
-                          "UI Merge",
-                          "A pointer to the GtkUIManager object that "
-                          "represents this pages menu hierarchy.",
-                          GTK_TYPE_UI_MANAGER,
-                          G_PARAM_READABLE));
-
-    g_object_class_install_property
-    (gobject_class,
-     PROP_ACTION_GROUP,
-     g_param_spec_object ("action-group",
-                          "Action Group",
-                          "A pointer to the GtkActionGroup object that "
-                          "represents this pages available menu/toolbar "
-                          "actions.",
-                          GTK_TYPE_ACTION_GROUP,
-                          G_PARAM_READABLE));
-
-
 
 
     signals[INSERTED] = g_signal_new ("inserted",
@@ -495,32 +454,37 @@ gnc_plugin_page_class_init (GncPluginPageClass *klass)
 
 
 /** Initialize a new instance of a gnucash content plugin.  This
- *  function initializes the object private storage space, and adds
- *  the object to the tracking system.
+ *  function initializes the object private storage space.
  *
  *  @param page The new object instance created by the object system.
- *
- *  @param klass A pointer to the class data structure for this
- *  object. */
+ *  */
 static void
-gnc_plugin_page_init (GncPluginPage *page, void *data)
+gnc_plugin_page_init (GncPluginPage *page)
 {
-    GncPluginPagePrivate *priv;
-    
-    GncPluginPageClass *klass = (GncPluginPageClass*)data;
-
-    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+    GncPluginPagePrivate *priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     priv->page_name   = NULL;
     priv->page_color  = NULL;
-    priv->uri         = NULL;
+    priv->page_changed_id = 0;
+    priv->focus_source_id = 0;
+    priv->menu_qualifier = NULL;
 
     page->window      = NULL;
     page->summarybar  = NULL;
-
-    gnc_gobject_tracking_remember(G_OBJECT(page), 
-		                  G_OBJECT_CLASS(klass));
 }
 
+/** The object has been fully constructed.
+ * This function adds the object to the tracking system.
+ *
+ *  @param obj The new object instance created by the object
+ *  system.
+ */
+ static void
+gnc_plugin_page_constructed (GObject *obj)
+{
+    gnc_gobject_tracking_remember(obj);
+
+    G_OBJECT_CLASS (gnc_plugin_page_parent_class)->constructed (obj);
+}
 
 /** Finalize the gnucash plugin object.  This function is called from
  *  the G_Object level to complete the destruction of the object.  It
@@ -535,28 +499,38 @@ gnc_plugin_page_finalize (GObject *object)
     GncPluginPagePrivate *priv;
     GncPluginPage *page;
 
-    page = GNC_PLUGIN_PAGE (object);
+    page = GNC_PLUGIN_PAGE(object);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+
+    if (priv->ui_description)
+        g_free (priv->ui_description);
+
     if (priv->page_name)
-        g_free(priv->page_name);
+        g_free (priv->page_name);
+
+    if (priv->page_long_name)
+        g_free (priv->page_long_name);
+
     if (priv->page_color)
-        g_free(priv->page_color);
-    if (priv->uri)
-        g_free(priv->uri);
+        g_free (priv->page_color);
+
     if (priv->statusbar_text)
-        g_free(priv->statusbar_text);
+        g_free (priv->statusbar_text);
 
     if (priv->books)
     {
-        g_list_free(priv->books);
+        g_list_free (priv->books);
         priv->books = NULL;
     }
 
+    if (priv->builder)
+        g_object_unref (priv->builder);
+
     page->window = NULL; // Don't need to free it.
 
-    gnc_gobject_tracking_forget(object);
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+    gnc_gobject_tracking_forget (object);
+    G_OBJECT_CLASS(gnc_plugin_page_parent_class)->finalize (object);
 }
 
 /************************************************************/
@@ -607,9 +581,6 @@ gnc_plugin_page_get_property (GObject     *object,
     case PROP_PAGE_COLOR:
         g_value_set_string (value, priv->page_color);
         break;
-    case PROP_PAGE_URI:
-        g_value_set_string (value, priv->uri);
-        break;
     case PROP_STATUSBAR_TEXT:
         g_value_set_string (value, priv->statusbar_text);
         break;
@@ -619,12 +590,6 @@ gnc_plugin_page_get_property (GObject     *object,
     case PROP_UI_DESCRIPTION:
         g_value_set_string (value, priv->ui_description);
         break;
-    case PROP_UI_MERGE:
-        g_value_take_object (value, priv->ui_merge);
-        break;
-    case PROP_ACTION_GROUP:
-        g_value_take_object (value, priv->action_group);
-        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -663,22 +628,19 @@ gnc_plugin_page_set_property (GObject      *object,
     switch (prop_id)
     {
     case PROP_PAGE_NAME:
-        gnc_plugin_page_set_page_name(page, g_value_get_string(value));
+        gnc_plugin_page_set_page_name (page, g_value_get_string (value));
         break;
     case PROP_PAGE_COLOR:
-        gnc_plugin_page_set_page_color(page, g_value_get_string(value));
-        break;
-    case PROP_PAGE_URI:
-        gnc_plugin_page_set_uri(page, g_value_get_string(value));
+        gnc_plugin_page_set_page_color (page, g_value_get_string (value));
         break;
     case PROP_STATUSBAR_TEXT:
-        gnc_plugin_page_set_statusbar_text(page, g_value_get_string(value));
+        gnc_plugin_page_set_statusbar_text (page, g_value_get_string (value));
         break;
     case PROP_USE_NEW_WINDOW:
-        gnc_plugin_page_set_use_new_window(page, g_value_get_boolean(value));
+        gnc_plugin_page_set_use_new_window (page, g_value_get_boolean (value));
         break;
     case PROP_UI_DESCRIPTION:
-        gnc_plugin_page_set_ui_description(page, g_value_get_string(value));
+        gnc_plugin_page_set_ui_description (page, g_value_get_string (value));
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -696,11 +658,11 @@ gnc_plugin_page_add_book (GncPluginPage *page, QofBook *book)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
     g_return_if_fail (book != NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    priv->books = g_list_append(priv->books, book);
+    priv->books = g_list_append (priv->books, book);
 }
 
 
@@ -711,11 +673,11 @@ gnc_plugin_page_has_book (GncPluginPage *page, QofBook *book)
     GncPluginPagePrivate *priv;
     GList *item;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), FALSE);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), FALSE);
     g_return_val_if_fail (book != NULL, FALSE);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    for (item = priv->books; item; item = g_list_next(item))
+    for (item = priv->books; item; item = g_list_next (item))
     {
         if (item->data == book)
         {
@@ -732,7 +694,7 @@ gnc_plugin_page_has_books (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), FALSE);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), FALSE);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return (priv->books != NULL);
@@ -744,7 +706,7 @@ gnc_plugin_page_has_books (GncPluginPage *page)
 GtkWidget *
 gnc_plugin_page_get_window (GncPluginPage *page)
 {
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     return page->window;
 }
@@ -757,7 +719,7 @@ gnc_plugin_page_get_page_name (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->page_name;
@@ -772,12 +734,13 @@ gnc_plugin_page_set_page_name (GncPluginPage *page, const gchar *name)
     GncPluginPagePrivate *priv;
     GncPluginPageClass *klass;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     if (priv->page_name)
-        g_free(priv->page_name);
-    priv->page_name = g_strdup(name);
+        g_free (priv->page_name);
+
+    priv->page_name = g_strdup (name);
 
     /* Perform page specific actions */
     klass = GNC_PLUGIN_PAGE_GET_CLASS (page);
@@ -796,7 +759,7 @@ gnc_plugin_page_get_page_long_name (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->page_long_name;
@@ -810,12 +773,13 @@ gnc_plugin_page_set_page_long_name (GncPluginPage *page, const gchar *name)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     if (priv->page_long_name)
-        g_free(priv->page_long_name);
-    priv->page_long_name = g_strdup(name);
+        g_free (priv->page_long_name);
+
+    priv->page_long_name = g_strdup (name);
 }
 
 
@@ -825,7 +789,7 @@ gnc_plugin_page_get_page_color (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->page_color;
@@ -838,41 +802,116 @@ gnc_plugin_page_set_page_color (GncPluginPage *page, const gchar *color)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     if (priv->page_color)
-        g_free(priv->page_color);
+        g_free (priv->page_color);
+
     if (color)
-        priv->page_color = g_strdup(color);
+        priv->page_color = g_strdup (color);
 }
 
 
-/*  Retrieve the Uniform Resource Identifier for this page. */
-const gchar *
-gnc_plugin_page_get_uri (GncPluginPage *page)
+static void
+gnc_plugin_page_focus_idle_destroy (GncPluginPage *plugin_page)
+{
+    GncPluginPagePrivate *priv = GNC_PLUGIN_PAGE_GET_PRIVATE(plugin_page);
+    priv->focus_source_id = 0;
+}
+
+
+static void
+gnc_plugin_page_default_focus (GncPluginPage *plugin_page,
+                               gboolean on_current_page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    if (!on_current_page)
+        return;
 
-    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    return priv->uri;
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(plugin_page));
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(plugin_page);
+
+    if (G_LIKELY(GNC_PLUGIN_PAGE_GET_CLASS(plugin_page)->focus_page_function))
+    {
+        // The page changed signal is emitted multiple times so we need
+        // to use an idle_add to change the focus
+
+        if (priv->focus_source_id > 0)
+            g_source_remove (priv->focus_source_id);
+
+        priv->focus_source_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                                    (GSourceFunc)(GNC_PLUGIN_PAGE_GET_CLASS(plugin_page)->focus_page_function),
+                                    GNC_PLUGIN_PAGE(plugin_page),
+                                    (GDestroyNotify)gnc_plugin_page_focus_idle_destroy);
+    }
 }
 
 
-/*  Set the Uniform Resource Identifier for this page. */
+/* this is the callback for the plugin "page_changed" signal */
+static void
+gnc_plugin_page_main_window_changed (GtkWindow *window,
+                                     GObject *object,
+                                     gpointer user_data)
+{
+    GncPluginPage *current_plugin_page = GNC_PLUGIN_PAGE(object);
+    GncPluginPage *plugin_page = GNC_PLUGIN_PAGE(user_data);
+    gboolean on_current_page = FALSE;
+
+    // Continue if current_plugin_page is valid
+    if (!current_plugin_page || !GNC_IS_PLUGIN_PAGE(current_plugin_page))
+        return;
+
+    // Continue only if the plugin_page is valid
+    if (!plugin_page || !GNC_IS_PLUGIN_PAGE(plugin_page))
+        return;
+
+    if (current_plugin_page == plugin_page)
+        on_current_page = TRUE;
+
+    (GNC_PLUGIN_PAGE_GET_CLASS(plugin_page)->focus_page)(plugin_page, on_current_page);
+}
+
+
+/* this is the callback for the plugin "inserted" signal which will setup
+ * the callback for the "page_changed" signal and save a pointer to the
+ * page focus function. */
 void
-gnc_plugin_page_set_uri (GncPluginPage *page, const gchar *name)
+gnc_plugin_page_inserted_cb (GncPluginPage *page, gpointer user_data)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    if (priv->uri)
-        g_free(priv->uri);
-    priv->uri = g_strdup(name);
+
+    priv->page_changed_id = g_signal_connect (G_OBJECT(page->window), "page_changed",
+                                              G_CALLBACK(gnc_plugin_page_main_window_changed),
+                                              page);
+
+    // on initial load try and set the page focus
+    if (!gnc_main_window_is_restoring_pages (GNC_MAIN_WINDOW (gnc_ui_get_main_window (page->window))))
+        (GNC_PLUGIN_PAGE_GET_CLASS(page)->focus_page)(page, TRUE);
+}
+
+
+/* disconnect the page_changed callback */
+void
+gnc_plugin_page_disconnect_page_changed (GncPluginPage *page)
+{
+    GncPluginPagePrivate *priv;
+
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+
+    if (priv->page_changed_id > 0)
+    {
+        g_signal_handler_disconnect (G_OBJECT(page->window), priv->page_changed_id);
+        priv->page_changed_id = 0;
+    }
 }
 
 
@@ -882,7 +921,7 @@ gnc_plugin_page_get_statusbar_text (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->statusbar_text;
@@ -895,12 +934,13 @@ gnc_plugin_page_set_statusbar_text (GncPluginPage *page, const gchar *message)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     if (priv->statusbar_text)
-        g_free(priv->statusbar_text);
-    priv->statusbar_text = g_strdup(message);
+        g_free (priv->statusbar_text);
+
+    priv->statusbar_text = g_strdup (message);
 }
 
 
@@ -910,7 +950,7 @@ gnc_plugin_page_get_use_new_window (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), FALSE);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), FALSE);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->use_new_window;
@@ -926,7 +966,7 @@ gnc_plugin_page_set_use_new_window (GncPluginPage *page, gboolean use_new)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail (GNC_IS_PLUGIN_PAGE (page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     priv->use_new_window = use_new;
@@ -939,7 +979,7 @@ gnc_plugin_page_get_ui_description (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), FALSE);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), FALSE);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     return priv->ui_description;
@@ -954,52 +994,111 @@ gnc_plugin_page_set_ui_description (GncPluginPage *page,
 {
     GncPluginPagePrivate *priv;
 
-    g_return_if_fail(GNC_IS_PLUGIN_PAGE(page));
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     if (priv->ui_description)
-        g_free(priv->ui_description);
-    priv->ui_description = g_strdup(ui_filename);
+        g_free (priv->ui_description);
+
+    priv->ui_description = g_strdup (ui_filename);
 }
 
 
-/*  Retrieve the GtkUIManager object associated with this page. */
-GtkUIManager *
-gnc_plugin_page_get_ui_merge (GncPluginPage *page)
+/*  Retrieve the GtkBuilder object associated with this page. */
+GtkBuilder *
+gnc_plugin_page_get_builder (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail(GNC_IS_PLUGIN_PAGE(page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    return priv->ui_merge;
+    return priv->builder;
 }
 
 
-/*  Retrieve the GtkActionGroup object associated with this page. */
-GtkActionGroup *
-gnc_plugin_page_get_action_group(GncPluginPage *page)
+/*  Retrieve the menu qualifier associated with this page. */
+const gchar *
+gnc_plugin_page_get_menu_qualifier (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
 
-    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE (page), NULL);
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
+
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    return priv->action_group;
+    return priv->menu_qualifier;
+}
+
+void
+gnc_plugin_page_set_menu_qualifier (GncPluginPage *page,
+                                    const char *menu_qualifier)
+{
+    GncPluginPagePrivate *priv;
+
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+
+    priv->menu_qualifier = menu_qualifier;
+}
+
+const gchar *
+gnc_plugin_page_get_menu_popup_qualifier (GncPluginPage *page)
+{
+    GncPluginPagePrivate *priv;
+
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+    return priv->menu_popup_qualifier;
+}
+
+void
+gnc_plugin_page_set_menu_popup_qualifier (GncPluginPage *page,
+                                    const char *menu_qualifier)
+{
+    GncPluginPagePrivate *priv;
+
+    g_return_if_fail (GNC_IS_PLUGIN_PAGE(page));
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+
+    priv->menu_popup_qualifier = menu_qualifier;
 }
 
 
-/*  Create the GtkActionGroup object associated with this page. */
-GtkActionGroup *
+GSimpleActionGroup *
+gnc_plugin_page_get_action_group (GncPluginPage *page)
+{
+    GncPluginPagePrivate *priv;
+
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
+
+    priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+    return priv->simple_action_group;
+}
+
+GSimpleActionGroup *
 gnc_plugin_page_create_action_group (GncPluginPage *page, const gchar *group_name)
+{
+    GncPluginPagePrivate *priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
+
+    priv->simple_action_group = g_simple_action_group_new ();
+    priv->simple_action_group_name = group_name;
+
+    return priv->simple_action_group;
+}
+
+const gchar *
+gnc_plugin_page_get_simple_action_group_name (GncPluginPage *page)
 {
     GncPluginPagePrivate *priv;
-    GtkActionGroup *group;
+
+    g_return_val_if_fail (GNC_IS_PLUGIN_PAGE(page), NULL);
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
-    group = gtk_action_group_new(group_name);
-    gtk_action_group_set_translation_domain(group, GETTEXT_PACKAGE);
-    priv->action_group = group;
-    return group;
+
+    return priv->simple_action_group_name;
 }
 
 gboolean
@@ -1007,11 +1106,13 @@ gnc_plugin_page_finish_pending (GncPluginPage *page)
 {
     if (!page)
         return TRUE;
+
     if (!GNC_IS_PLUGIN_PAGE(page))
         return TRUE;
 
     if (!GNC_PLUGIN_PAGE_GET_CLASS(page)->finish_pending)
         return TRUE;
+
     return (GNC_PLUGIN_PAGE_GET_CLASS(page)->finish_pending)(page);
 }
 
