@@ -61,10 +61,13 @@ enum
 #define MIN_BUTT_WIDTH 20 // minimum size for a button excluding border
 
 static QofLogModule log_module = G_LOG_DOMAIN;
-static GtkBoxClass *gnc_item_edit_parent_class;
 
-static GtkToggleButtonClass *gnc_item_edit_tb_parent_class;
 static void gnc_item_edit_destroying (GtkWidget *this, gpointer data);
+
+G_DEFINE_TYPE (GncItemEdit, gnc_item_edit, GTK_TYPE_BOX)
+
+G_DEFINE_TYPE (GncItemEditTb, gnc_item_edit_tb, GTK_TYPE_TOGGLE_BUTTON)
+
 static void
 gnc_item_edit_tb_init (GncItemEditTb *item_edit_tb)
 {
@@ -151,8 +154,6 @@ gnc_item_edit_tb_class_init (GncItemEditTbClass *gnc_item_edit_tb_class)
 
     gtk_widget_class_set_css_name (GTK_WIDGET_CLASS(gnc_item_edit_tb_class), "button");
 
-    gnc_item_edit_tb_parent_class = g_type_class_peek_parent (gnc_item_edit_tb_class);
-
     object_class = G_OBJECT_CLASS(gnc_item_edit_tb_class);
     widget_class = GTK_WIDGET_CLASS(gnc_item_edit_tb_class);
 
@@ -172,34 +173,6 @@ gnc_item_edit_tb_class_init (GncItemEditTbClass *gnc_item_edit_tb_class)
     widget_class->get_preferred_height = gnc_item_edit_tb_get_preferred_height;
 }
 
-GType
-gnc_item_edit_tb_get_type (void)
-{
-    static GType gnc_item_edit_tb_type = 0;
-
-    if (!gnc_item_edit_tb_type)
-    {
-        static const GTypeInfo gnc_item_edit_tb_info =
-        {
-            sizeof (GncItemEditTbClass),
-            NULL,
-            NULL,
-            (GClassInitFunc)gnc_item_edit_tb_class_init,
-            NULL,
-            NULL,
-            sizeof (GncItemEditTb),
-            0, /* n_preallocs */
-            (GInstanceInitFunc)gnc_item_edit_tb_init,
-            NULL,
-        };
-        gnc_item_edit_tb_type =
-            g_type_register_static (GTK_TYPE_TOGGLE_BUTTON,
-                                    "GncItemEditTb",
-                                    &gnc_item_edit_tb_info, 0);
-    }
-    return gnc_item_edit_tb_type;
-}
-
 GtkWidget *
 gnc_item_edit_tb_new (GnucashSheet *sheet)
 {
@@ -346,6 +319,7 @@ gnc_item_edit_init (GncItemEdit *item_edit)
     item_edit->popup_user_data = NULL;
     item_edit->popup_returned_height = 0;
     item_edit->popup_height_signal_id = 0;
+    item_edit->popup_allocation_height = -1;
 
     item_edit->style = NULL;
     item_edit->button_width = MIN_BUTT_WIDTH;
@@ -775,8 +749,6 @@ gnc_item_edit_class_init (GncItemEditClass *gnc_item_edit_class)
 
     gtk_widget_class_set_css_name (GTK_WIDGET_CLASS(gnc_item_edit_class), "gnc-id-cursor");
 
-    gnc_item_edit_parent_class = g_type_class_peek_parent (gnc_item_edit_class);
-
     object_class = G_OBJECT_CLASS(gnc_item_edit_class);
     widget_class = GTK_WIDGET_CLASS(gnc_item_edit_class);
 
@@ -796,39 +768,6 @@ gnc_item_edit_class_init (GncItemEditClass *gnc_item_edit_class)
     widget_class->get_preferred_height = gnc_item_edit_get_preferred_height;
 }
 
-/* FIXME: This way of initializing GObjects is obsolete. We should be
- * using G_DECLARE_FINAL_TYPE instead of rolling _get_type by hand.
- */
-GType
-gnc_item_edit_get_type (void)
-{
-    static GType gnc_item_edit_type = 0;
-
-    if (!gnc_item_edit_type)
-    {
-        static const GTypeInfo gnc_item_edit_info =
-        {
-            sizeof (GncItemEditClass),
-            NULL,
-            NULL,
-            (GClassInitFunc) gnc_item_edit_class_init,
-            NULL,
-            NULL,
-            sizeof (GncItemEdit),
-            0, /* n_preallocs */
-            (GInstanceInitFunc) gnc_item_edit_init,
-            NULL,
-        };
-
-        gnc_item_edit_type =
-            g_type_register_static (GTK_TYPE_BOX,
-                                    "GncItemEdit",
-                                    &gnc_item_edit_info, 0);
-    }
-
-    return gnc_item_edit_type;
-}
-
 gint
 gnc_item_edit_get_margin (GncItemEdit *item_edit, Sides side)
 {
@@ -913,6 +852,10 @@ button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer *pointer)
         }
         return TRUE;
     }
+
+    if (!gtk_widget_has_focus (GTK_WIDGET(sheet)))
+        gtk_widget_grab_focus (GTK_WIDGET(sheet));
+
     return FALSE;
 }
 
@@ -1006,15 +949,15 @@ gnc_item_edit_destroying (GtkWidget *item_edit, gpointer data)
 static void
 check_popup_height_is_true (GtkWidget    *widget,
                             GdkRectangle *allocation,
-                            gpointer user_data)
+                            gpointer      user_data)
 {
     GncItemEdit *item_edit = GNC_ITEM_EDIT(user_data);
 
-    // if a larger font is specified in css for the sheet, the popup returned height value
-    // on first pop does not reflect the true height but the minimum height so just to be
-    // sure check this value against the allocated one.
+    // the popup returned height value on first pop sometimes does not reflect the true height
+    // but the minimum height so just to be sure check this value against the allocated one.
     if (allocation->height != item_edit->popup_returned_height)
     {
+        item_edit->popup_allocation_height = allocation->height;
         gtk_container_remove (GTK_CONTAINER(item_edit->sheet), item_edit->popup_item);
 
         g_idle_add_full (G_PRIORITY_HIGH_IDLE,
@@ -1167,6 +1110,8 @@ gnc_item_edit_hide_popup (GncItemEdit *item_edit)
     gtk_toggle_button_set_active
         (GTK_TOGGLE_BUTTON(item_edit->popup_toggle.tbutton), FALSE);
 
+    item_edit->popup_allocation_height = -1;
+
     gtk_widget_grab_focus (GTK_WIDGET(item_edit->sheet));
 }
 
