@@ -50,19 +50,19 @@
          (q-start-date (NDayDelta q-start-date 5)))
 
     (let* ((accounts (env-create-account-structure-alist env (list "Assets"
-								   (list (cons 'type ACCT-TYPE-ASSET))
-								   (list "Bank Account")
-								   (list "Wallet"))))
-	   (bank-account (cdr (assoc "Bank Account" accounts)))
-	   (wallet (cdr (assoc "Wallet" accounts))))
+                                                                   (list (cons 'type ACCT-TYPE-ASSET))
+                                                                   (list "Bank Account")
+                                                                   (list "Wallet"))))
+           (bank-account (cdr (assoc "Bank Account" accounts)))
+           (wallet (cdr (assoc "Wallet" accounts))))
 
       (env-create-daily-transactions env start-date end-date bank-account wallet)
       (format #t "Created transactions for each day from ~a to ~a~%" (gnc-ctime start-date) (gnc-ctime end-date))
       (let ((splits (gnc:account-get-trans-type-splits-interval (list bank-account wallet)
-							      ACCT-TYPE-ASSET
-							      q-start-date q-end-date)))
-	;; 10 is the right number (5 days, two splits per tx)
-	(test-equal "length splits = 10"
+                                                              ACCT-TYPE-ASSET
+                                                              q-start-date q-end-date)))
+        ;; 10 is the right number (5 days, two splits per tx)
+        (test-equal "length splits = 10"
           10
           (length splits)))))
   (teardown)))
@@ -261,6 +261,17 @@
       (coll-A 'add GBP -1)
       (test-equal "gnc-commodity-collector does not round inappropriately"
         '(("GBP" . 0))
+        (collector->list coll-A))
+
+      ;; the following tests the remove-zeros action
+      (coll-A 'add USD 1)
+      (coll-A 'add EUR 0)
+      (test-equal "gnc-commodity-collector before remove-zeros"
+        '(("EUR" . 0) ("USD" . 1) ("GBP" . 0))
+        (collector->list coll-A))
+      (coll-A 'remove-zeros #f #f)
+      (test-equal "gnc-commodity-collector after remove-zeros"
+        '(("USD" . 1))
         (collector->list coll-A)))
     (teardown)))
 
@@ -472,8 +483,8 @@
           (collector->list
            (gnc:select-assoc-account-balance account-balances bank)))
 
-        (test-equal "gnc:select-assoc-account-balance - miss"
-          #f
+        (test-error "gnc:select-assoc-account-balance - miss"
+          'wrong-type-arg
           (collector->list
            (gnc:select-assoc-account-balance account-balances expense)))
 
@@ -601,6 +612,7 @@
                                   (list "Bank1")
                                   (list "Bank2")
                                   (list "Bank3")
+                                  (list "Bank5")
                                   (list "Bank4"))
                             (list "Income" (list (cons 'type ACCT-TYPE-INCOME)))))
            (accounts (env-create-account-structure-alist env structure))
@@ -608,6 +620,7 @@
            (bank2 (assoc-ref accounts "Bank2"))
            (bank3 (assoc-ref accounts "Bank3"))
            (bank4 (assoc-ref accounts "Bank4"))
+           (bank5 (assoc-ref accounts "Bank5"))
            (income (assoc-ref accounts "Income"))
            (dates (gnc:make-date-list (gnc-dmy2time64 01 01 1970)
                                       (gnc-dmy2time64 01 04 1970)
@@ -681,6 +694,15 @@
         '(#f 18 18 18)
         (gnc:account-accumulate-at-dates bank4 dates))
 
+      ;; create 3 transactions but modify posted dates to exact time64 numbers
+      (xaccTransSetDatePostedSecs (env-transfer env 1 1 2000 income bank5 2) 200)
+      (xaccTransSetDatePostedSecs (env-transfer env 1 1 2000 income bank5 3) 300)
+      (xaccTransSetDatePostedSecs (env-transfer env 1 1 2000 income bank5 5) 400)
+
+      (test-equal "transactions at date boundaries"
+                  '(#f 2 5 10 10)
+                  (gnc:account-accumulate-at-dates bank5 (list 100 200 300 400 500)))
+
       ;; Tests split->date sorting. note the 3 txns created below are
       ;; initially sorted by posted_date ie txn2 < txn3 <
       ;; txn1. However the reconciled_date sorting will be
