@@ -25,8 +25,9 @@
  * This file implements the top-level QofBackend API for saving/
  * restoring data to/from an SQL db using libdbi
  */
-extern "C"
-{
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "config.h"
 
 #include <platform.h>
@@ -57,11 +58,6 @@ extern "C"
 #include "splint-defs.h"
 #endif
 
-}
-
-#include <glib.h>
-#include <glib/gstdio.h>
-
 #include <boost/regex.hpp>
 #include <string>
 #include <iomanip>
@@ -182,7 +178,7 @@ UriStrings::quote_dbname(DbType t) const noexcept
 static void
 set_options(dbi_conn conn, const PairVec& options)
 {
-    for (auto option : options)
+    for (const auto& option : options)
     {
         auto opt = option.first.c_str();
         auto val = option.second.c_str();
@@ -209,7 +205,6 @@ GncDbiBackend<Type>::set_standard_connection_options (dbi_conn conn,
                                                 const UriStrings& uri)
 
 {
-    gint result;
     PairVec options;
     options.push_back(std::make_pair("host", uri.m_host));
     options.push_back(std::make_pair("dbname", uri.m_dbname));
@@ -224,7 +219,7 @@ GncDbiBackend<Type>::set_standard_connection_options (dbi_conn conn,
         {
             const char *msg = nullptr;
             auto err = dbi_conn_error(conn, &msg);
-            PERR("Error setting port option to %d: %s", uri.m_portnum, msg);
+            PERR("Error (%d) setting port option to %d: %s", err, uri.m_portnum, msg);
             throw std::runtime_error(msg);
         }
     }
@@ -500,7 +495,7 @@ error_handler<DbType::DBI_MYSQL> (dbi_conn conn, void* user_data)
     if (!dbi_be->connected())
     {
         PINFO ("DBI error: %s\n", msg);
-        PINFO ("Note: GbcDbiSqlConnection not yet initialized. Skipping further error processing.");
+        PINFO ("Note: GncDbiSqlConnection not yet initialized. Skipping further error processing.");
         return;
     }
 
@@ -642,7 +637,6 @@ template <DbType Type> void
 GncDbiBackend<Type>::session_begin (QofSession* session, const char* new_uri,
                                     SessionOpenMode mode)
 {
-    GncDbiTestResult dbi_test_result = GNC_DBI_PASS;
     PairVec options;
 
     g_return_if_fail (session != nullptr);
@@ -714,7 +708,7 @@ GncDbiBackend<Type>::session_begin (QofSession* session, const char* new_uri,
     else if (m_exists)
     {
         PERR ("Unable to connect to database '%s'\n", uri.dbname());
-        set_error (ERR_BACKEND_SERVER_ERR);
+        set_error (ERR_BACKEND_CANT_CONNECT);
         dbi_conn_close(conn);
         LEAVE("Error");
         return;
@@ -982,8 +976,8 @@ GncDbiBackend<Type>::safe_sync (QofBook* book)
         LEAVE ("Failed to create new database tables");
         return;
     }
-    conn->table_operation (TableOpType::drop_backup);
     conn->commit_transaction();
+    conn->table_operation (TableOpType::drop_backup);
     LEAVE ("book=%p", m_book);
 }
 /* MySQL commits the transaction and all savepoints after the first CREATE
@@ -1036,7 +1030,7 @@ template<> bool
 QofDbiBackendProvider<DbType::DBI_SQLITE>::type_check(const char *uri)
 {
     FILE* f;
-    gchar buf[50];
+    gchar buf[51]{};
     G_GNUC_UNUSED size_t chars_read;
     gint status;
     gchar* filename;
@@ -1056,7 +1050,7 @@ QofDbiBackendProvider<DbType::DBI_SQLITE>::type_check(const char *uri)
     }
 
     // OK if file has the correct header
-    chars_read = fread (buf, sizeof (buf), 1, f);
+    chars_read = fread (buf, sizeof (buf) - 1, 1, f);
     status = fclose (f);
     if (status < 0)
     {
@@ -1100,10 +1094,17 @@ gnc_module_init_backend_dbi (void)
 #endif
     if (num_drivers <= 0)
     {
-        gchar* dir = g_build_filename (gnc_path_get_libdir (), "dbd", nullptr);
 #if HAVE_LIBDBI_R
         if (dbi_instance)
-            return;
+        {
+            dbi_shutdown_r (dbi_instance);
+            dbi_instance = nullptr;
+        }
+#endif
+        gchar *libdir = gnc_path_get_libdir ();
+        gchar *dir = g_build_filename (libdir, "dbd", nullptr);
+        g_free (libdir);
+#if HAVE_LIBDBI_R
         num_drivers = dbi_initialize_r (dir, &dbi_instance);
 #else
         num_drivers = dbi_initialize (dir);
@@ -1207,6 +1208,21 @@ gnc_module_finalize_backend_dbi (void)
 
 /* --------------------------------------------------------- */
 
+static void
+log_failed_field(dbi_result result, const char* fieldname)
+{
+    auto idx = dbi_result_get_field_idx(result, fieldname);
+    if (dbi_result_field_is_null_idx(result, idx))
+        PERR("Result field %s is NULL", fieldname);
+    else
+    {
+        auto type = dbi_result_get_field_type_idx(result, idx);
+        auto attribs = dbi_result_get_field_attribs_idx(result, idx);
+        PERR("Result field %s has type %d and attribs %d",
+             fieldname, type, attribs);
+    }
+}
+
 /** Users discovered a bug in some distributions of libdbi, where if
  * it is compiled on certain versions of gcc with the -ffast-math
  * compiler option it fails to correctly handle saving of 64-bit
@@ -1248,7 +1264,7 @@ dbi_library_test (dbi_conn conn)
     dbi_result_free (result);
     auto locale = gnc_push_locale (LC_NUMERIC, "C");
     result = dbi_conn_query (conn, "SELECT * FROM numtest");
-    if (result == nullptr)
+    if (result == nullptr || !dbi_result_get_numrows(result))
     {
         const char* errmsg;
         dbi_conn_error (conn, &errmsg);
@@ -1261,8 +1277,14 @@ dbi_library_test (dbi_conn conn)
     while (dbi_result_next_row (result))
     {
         resultlonglong = dbi_result_get_longlong (result, "test_int");
+        if (!resultlonglong)
+            log_failed_field(result, "test_int");
         resultulonglong = dbi_result_get_ulonglong (result, "test_unsigned");
+        if (!resultulonglong)
+            log_failed_field(result, "test_unsigned");
         resultdouble = dbi_result_get_double (result, "test_double");
+        if (!resultdouble)
+            log_failed_field(result, "test_double");
     }
     dbi_conn_query (conn, "DROP TABLE numtest");
     gnc_pop_locale (LC_NUMERIC, locale);
