@@ -66,7 +66,7 @@ const gchar* employee_version_string = "2.0.0";
 static void
 maybe_add_string (xmlNodePtr ptr, const char* tag, const char* str)
 {
-    if (str && strlen (str) > 0)
+    if (str && *str)
         xmlAddChild (ptr, text_to_dom_tree (tag, str));
 }
 
@@ -129,40 +129,25 @@ struct employee_pdata
     QofBook* book;
 };
 
-static gboolean
-set_string (xmlNodePtr node, GncEmployee* employee,
-            void (*func) (GncEmployee* employee, const char* txt))
-{
-    char* txt = dom_tree_to_text (node);
-    g_return_val_if_fail (txt, FALSE);
-
-    func (employee, txt);
-
-    g_free (txt);
-
-    return TRUE;
-}
-
 static gboolean
 employee_username_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
 
-    return set_string (node, pdata->employee, gncEmployeeSetUsername);
+    return apply_xmlnode_text (gncEmployeeSetUsername, pdata->employee, node);
 }
 
 static gboolean
 employee_guid_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
-    GncGUID* guid;
     GncEmployee* employee;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
 
     /* See if we've already created this one */
-    employee = gncEmployeeLookup (pdata->book, guid);
+    employee = gncEmployeeLookup (pdata->book, &*guid);
     if (employee)
     {
         gncEmployeeDestroy (pdata->employee);
@@ -171,11 +156,9 @@ employee_guid_handler (xmlNodePtr node, gpointer employee_pdata)
     }
     else
     {
-        gncEmployeeSetGUID (pdata->employee, guid);
+        gncEmployeeSetGUID (pdata->employee, &*guid);
     }
 
-    guid_free (guid);
-
     return TRUE;
 }
 
@@ -184,7 +167,7 @@ employee_id_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
 
-    return set_string (node, pdata->employee, gncEmployeeSetID);
+    return apply_xmlnode_text (gncEmployeeSetID, pdata->employee, node);
 }
 
 static gboolean
@@ -192,7 +175,7 @@ employee_language_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
 
-    return set_string (node, pdata->employee, gncEmployeeSetLanguage);
+    return apply_xmlnode_text (gncEmployeeSetLanguage, pdata->employee, node);
 }
 
 static gboolean
@@ -200,7 +183,7 @@ employee_acl_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
 
-    return set_string (node, pdata->employee, gncEmployeeSetAcl);
+    return apply_xmlnode_text (gncEmployeeSetAcl, pdata->employee, node);
 }
 
 static gboolean
@@ -229,13 +212,8 @@ static gboolean
 employee_workday_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
-    gnc_numeric* val;
-
-    val = dom_tree_to_gnc_numeric (node);
-    g_return_val_if_fail (val, FALSE);
-    gncEmployeeSetWorkday (pdata->employee, *val);
-    g_free (val);
 
+    gncEmployeeSetWorkday (pdata->employee, dom_tree_to_gnc_numeric (node));
     return TRUE;
 }
 
@@ -243,13 +221,8 @@ static gboolean
 employee_rate_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
-    gnc_numeric* val;
-
-    val = dom_tree_to_gnc_numeric (node);
-    g_return_val_if_fail (val, FALSE);
-    gncEmployeeSetRate (pdata->employee, *val);
-    g_free (val);
 
+    gncEmployeeSetRate (pdata->employee, dom_tree_to_gnc_numeric (node));
     return TRUE;
 }
 
@@ -271,14 +244,12 @@ static gboolean
 employee_ccard_handler (xmlNodePtr node, gpointer employee_pdata)
 {
     struct employee_pdata* pdata = static_cast<decltype (pdata)> (employee_pdata);
-    GncGUID* guid;
     Account* ccard_acc;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
 
-    ccard_acc = xaccAccountLookup (guid, pdata->book);
-    guid_free (guid);
+    ccard_acc = xaccAccountLookup (&*guid, pdata->book);
 
     g_return_val_if_fail (ccard_acc, FALSE);
     gncEmployeeSetCCard (pdata->employee, ccard_acc);
