@@ -93,7 +93,7 @@ const gchar* entry_version_string = "2.0.0";
 static void
 maybe_add_string (xmlNodePtr ptr, const char* tag, const char* str)
 {
-    if (str && strlen (str) > 0)
+    if (str && *str)
         xmlAddChild (ptr, text_to_dom_tree (tag, str));
 }
 
@@ -224,18 +224,6 @@ struct entry_pdata
     Account* acc;
 };
 
-static inline gboolean
-set_string (xmlNodePtr node, GncEntry* entry,
-            void (*func) (GncEntry* entry, const char* txt))
-{
-    char* txt = dom_tree_to_text (node);
-    g_return_val_if_fail (txt, FALSE);
-
-    func (entry, txt);
-    g_free (txt);
-    return TRUE;
-}
-
 static inline gboolean
 set_time64 (xmlNodePtr node, GncEntry* entry,
               void (*func) (GncEntry* entry, time64 ts))
@@ -250,11 +238,7 @@ static inline gboolean
 set_numeric (xmlNodePtr node, GncEntry* entry,
              void (*func) (GncEntry* entry, gnc_numeric num))
 {
-    gnc_numeric* num = dom_tree_to_gnc_numeric (node);
-    g_return_val_if_fail (num, FALSE);
-
-    func (entry, *num);
-    g_free (num);
+    func (entry, dom_tree_to_gnc_numeric (node));
     return TRUE;
 }
 
@@ -274,13 +258,11 @@ static inline gboolean
 set_account (xmlNodePtr node, struct entry_pdata* pdata,
              void (*func) (GncEntry* entry, Account* acc))
 {
-    GncGUID* guid;
     Account* acc;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    acc = xaccAccountLookup (guid, pdata->book);
-    guid_free (guid);
+    acc = xaccAccountLookup (&*guid, pdata->book);
     g_return_val_if_fail (acc, FALSE);
 
     if (func)
@@ -294,24 +276,22 @@ static inline gboolean
 set_taxtable (xmlNodePtr node, struct entry_pdata* pdata,
               void (*func) (GncEntry* entry, GncTaxTable* taxtable))
 {
-    GncGUID* guid;
     GncTaxTable* taxtable;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    taxtable = gncTaxTableLookup (pdata->book, guid);
+    taxtable = gncTaxTableLookup (pdata->book, &*guid);
     if (!taxtable)
     {
         taxtable = gncTaxTableCreate (pdata->book);
         gncTaxTableBeginEdit (taxtable);
-        gncTaxTableSetGUID (taxtable, guid);
+        gncTaxTableSetGUID (taxtable, &*guid);
         gncTaxTableCommitEdit (taxtable);
     }
     else
         gncTaxTableDecRef (taxtable);
 
     func (pdata->entry, taxtable);
-    guid_free (guid);
     return TRUE;
 }
 
@@ -319,12 +299,11 @@ static gboolean
 entry_guid_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncGUID* guid;
     GncEntry* entry;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    entry = gncEntryLookup (pdata->book, guid);
+    entry = gncEntryLookup (pdata->book, &*guid);
     if (entry)
     {
         gncEntryDestroy (pdata->entry);
@@ -333,11 +312,9 @@ entry_guid_handler (xmlNodePtr node, gpointer entry_pdata)
     }
     else
     {
-        gncEntrySetGUID (pdata->entry, guid);
+        gncEntrySetGUID (pdata->entry, &*guid);
     }
 
-    guid_free (guid);
-
     return TRUE;
 }
 
@@ -360,7 +337,7 @@ entry_description_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
 
-    return set_string (node, pdata->entry, gncEntrySetDescription);
+    return apply_xmlnode_text (gncEntrySetDescription, pdata->entry, node);
 }
 
 static gboolean
@@ -368,7 +345,7 @@ entry_action_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
 
-    return set_string (node, pdata->entry, gncEntrySetAction);
+    return apply_xmlnode_text (gncEntrySetAction, pdata->entry, node);
 }
 
 static gboolean
@@ -376,7 +353,7 @@ entry_notes_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
 
-    return set_string (node, pdata->entry, gncEntrySetNotes);
+    return apply_xmlnode_text (gncEntrySetNotes, pdata->entry, node);
 }
 
 static gboolean
@@ -416,40 +393,30 @@ static gboolean
 entry_idisctype_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncAmountType type;
-    char* str;
-    gboolean ret;
-
-    str = dom_tree_to_text (node);
-    g_return_val_if_fail (str, FALSE);
-
-    ret = gncAmountStringToType (str, &type);
-    g_free (str);
-
-    if (ret)
-        gncEntrySetInvDiscountType (pdata->entry, type);
-
-    return ret;
+    auto entry = pdata->entry;
+    auto set_discount_type = [entry](auto str)
+    {
+        GncAmountType type;
+        if (!gncAmountStringToType (str, &type)) return false;
+        gncEntrySetInvDiscountType (entry, type);
+        return true;
+    };
+    return apply_xmlnode_text (set_discount_type, node, FALSE);
 }
 
 static gboolean
 entry_idischow_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncDiscountHow how;
-    char* str;
-    gboolean ret;
-
-    str = dom_tree_to_text (node);
-    g_return_val_if_fail (str, FALSE);
-
-    ret = gncEntryDiscountStringToHow (str, &how);
-    g_free (str);
-
-    if (ret)
-        gncEntrySetInvDiscountHow (pdata->entry, how);
-
-    return ret;
+    auto entry = pdata->entry;
+    auto set_discount_how = [entry](auto str)
+    {
+        GncDiscountHow how;
+        if (!gncEntryDiscountStringToHow (str, &how)) return false;
+        gncEntrySetInvDiscountHow (entry, how);
+        return true;
+    };
+    return apply_xmlnode_text (set_discount_how, node, FALSE);
 }
 
 static gboolean
@@ -537,20 +504,15 @@ static gboolean
 entry_billpayment_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncEntryPaymentType type;
-    char* str;
-    gboolean ret;
-
-    str = dom_tree_to_text (node);
-    g_return_val_if_fail (str, FALSE);
-
-    ret = gncEntryPaymentStringToType (str, &type);
-    g_free (str);
-
-    if (ret)
-        gncEntrySetBillPayment (pdata->entry, type);
-
-    return ret;
+    auto entry = pdata->entry;
+    auto set_billpayment = [entry](auto str)
+    {
+        GncEntryPaymentType type;
+        if (!gncEntryPaymentStringToType (str, &type)) return false;
+        gncEntrySetBillPayment (entry, type);
+        return true;
+    };
+    return apply_xmlnode_text (set_billpayment, node, FALSE);
 }
 
 /* The rest of the stuff */
@@ -559,24 +521,22 @@ static gboolean
 entry_order_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncGUID* guid;
     GncOrder* order;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    order = gncOrderLookup (pdata->book, guid);
+    order = gncOrderLookup (pdata->book, &*guid);
     if (!order)
     {
         order = gncOrderCreate (pdata->book);
         gncOrderBeginEdit (order);
-        gncOrderSetGUID (order, guid);
+        gncOrderSetGUID (order, &*guid);
         gncOrderCommitEdit (order);
     }
     gncOrderBeginEdit (order);
     gncOrderAddEntry (order, pdata->entry);
     gncOrderCommitEdit (order);
 
-    guid_free (guid);
     return TRUE;
 }
 
@@ -584,24 +544,22 @@ static gboolean
 entry_invoice_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncGUID* guid;
     GncInvoice* invoice;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    invoice = gncInvoiceLookup (pdata->book, guid);
+    invoice = gncInvoiceLookup (pdata->book, &*guid);
     if (!invoice)
     {
         invoice = gncInvoiceCreate (pdata->book);
         gncInvoiceBeginEdit (invoice);
-        gncInvoiceSetGUID (invoice, guid);
+        gncInvoiceSetGUID (invoice, &*guid);
         gncInvoiceCommitEdit (invoice);
     }
     gncInvoiceBeginEdit (invoice);
     gncInvoiceAddEntry (invoice, pdata->entry);
     gncInvoiceCommitEdit (invoice);
 
-    guid_free (guid);
     return TRUE;
 }
 
@@ -609,24 +567,22 @@ static gboolean
 entry_bill_handler (xmlNodePtr node, gpointer entry_pdata)
 {
     struct entry_pdata* pdata = static_cast<decltype (pdata)> (entry_pdata);
-    GncGUID* guid;
     GncInvoice* invoice;
 
-    guid = dom_tree_to_guid (node);
+    auto guid = dom_tree_to_guid (node);
     g_return_val_if_fail (guid, FALSE);
-    invoice = gncInvoiceLookup (pdata->book, guid);
+    invoice = gncInvoiceLookup (pdata->book, &*guid);
     if (!invoice)
     {
         invoice = gncInvoiceCreate (pdata->book);
         gncInvoiceBeginEdit (invoice);
-        gncInvoiceSetGUID (invoice, guid);
+        gncInvoiceSetGUID (invoice, &*guid);
         gncInvoiceCommitEdit (invoice);
     }
     gncInvoiceBeginEdit (invoice);
     gncBillAddEntry (invoice, pdata->entry);
     gncInvoiceCommitEdit (invoice);
 
-    guid_free (guid);
     return TRUE;
 }
 
