@@ -91,10 +91,9 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
     gnc_price_begin_edit (p);
     if (g_strcmp0 ("price:id", (char*)sub_node->name) == 0)
     {
-        GncGUID* c = dom_tree_to_guid (sub_node);
+        auto c = dom_tree_to_guid (sub_node);
         if (!c) return FALSE;
-        gnc_price_set_guid (p, c);
-        guid_free (c);
+        gnc_price_set_guid (p, &*c);
     }
     else if (g_strcmp0 ("price:commodity", (char*)sub_node->name) == 0)
     {
@@ -116,24 +115,17 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
     }
     else if (g_strcmp0 ("price:source", (char*)sub_node->name) == 0)
     {
-        char* text = dom_tree_to_text (sub_node);
-        if (!text) return FALSE;
-        gnc_price_set_source_string (p, text);
-        g_free (text);
+        if (!apply_xmlnode_text (gnc_price_set_source_string, p, sub_node))
+            return FALSE;
     }
     else if (g_strcmp0 ("price:type", (char*)sub_node->name) == 0)
     {
-        char* text = dom_tree_to_text (sub_node);
-        if (!text) return FALSE;
-        gnc_price_set_typestr (p, text);
-        g_free (text);
+        if (!apply_xmlnode_text (gnc_price_set_typestr, p, sub_node))
+            return FALSE;
     }
     else if (g_strcmp0 ("price:value", (char*)sub_node->name) == 0)
     {
-        gnc_numeric* value = dom_tree_to_gnc_numeric (sub_node);
-        if (!value) return FALSE;
-        gnc_price_set_value (p, *value);
-        g_free (value);
+        gnc_price_set_value (p, dom_tree_to_gnc_numeric (sub_node));
     }
     gnc_price_commit_edit (p);
     return TRUE;
@@ -439,14 +431,14 @@ gnc_price_to_dom_tree (const xmlChar* tag, GNCPrice* price)
     if (!add_child_or_kill_parent (price_xml, tmpnode)) return NULL;
 
     sourcestr = gnc_price_get_source_string (price);
-    if (sourcestr && (strlen (sourcestr) != 0))
+    if (sourcestr && *sourcestr)
     {
         tmpnode = text_to_dom_tree ("price:source", sourcestr);
         if (!add_child_or_kill_parent (price_xml, tmpnode)) return NULL;
     }
 
     typestr = gnc_price_get_typestr (price);
-    if (typestr && (strlen (typestr) != 0))
+    if (typestr && *typestr)
     {
         tmpnode = text_to_dom_tree ("price:type", typestr);
         if (!add_child_or_kill_parent (price_xml, tmpnode)) return NULL;
