@@ -34,11 +34,11 @@
 
 #include <gnc-xml-helper.h>
 #include <Account.h>
-#include <AccountP.h>
+#include <AccountP.hpp>
 #include <Query.h>
 #include <Scrub.h>
 #include <Transaction.h>
-#include <TransactionP.h>
+#include <TransactionP.hpp>
 #include <TransLog.h>
 #include <gnc-pricedb.h>
 #include <gnc-pricedb-p.h>
@@ -472,6 +472,13 @@ simple_kvp_value_parser_new (sixtp_end_handler end_handler)
 
  */
 
+static gboolean
+string_to_gnc_numeric(const gchar* str, gnc_numeric *n)
+{
+    *n = gnc_numeric_from_string (str);
+    return (!gnc_numeric_check (*n));
+}
+
 /* ------------------------------------------------------------ */
 /* generic type copnversion for kvp types */
 #define KVP_CVT_VALUE(TYPE)                 \
@@ -2928,10 +2935,9 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
 
     if (g_strcmp0 ("price:id", (char*)sub_node->name) == 0)
     {
-        GncGUID* c = dom_tree_to_guid (sub_node);
+        auto c = dom_tree_to_guid (sub_node);
         if (!c) return FALSE;
-        gnc_price_set_guid (p, c);
-        guid_free (c);
+        gnc_price_set_guid (p, &*c);
     }
     else if (g_strcmp0 ("price:commodity", (char*)sub_node->name) == 0)
     {
@@ -2953,24 +2959,17 @@ price_parse_xml_sub_node (GNCPrice* p, xmlNodePtr sub_node, QofBook* book)
     }
     else if (g_strcmp0 ("price:source", (char*)sub_node->name) == 0)
     {
-        char* text = dom_tree_to_text (sub_node);
-        if (!text) return FALSE;
-        gnc_price_set_source_string (p, text);
-        g_free (text);
+        if (!apply_xmlnode_text (gnc_price_set_source_string, p, sub_node))
+            return false;
     }
     else if (g_strcmp0 ("price:type", (char*)sub_node->name) == 0)
     {
-        char* text = dom_tree_to_text (sub_node);
-        if (!text) return FALSE;
-        gnc_price_set_typestr (p, text);
-        g_free (text);
+        if (!apply_xmlnode_text (gnc_price_set_typestr, p, sub_node))
+            return false;
     }
     else if (g_strcmp0 ("price:value", (char*)sub_node->name) == 0)
     {
-        gnc_numeric* value = dom_tree_to_gnc_numeric (sub_node);
-        if (!value) return FALSE;
-        gnc_price_set_value (p, *value);
-        g_free (value);
+        gnc_price_set_value (p, dom_tree_to_gnc_numeric (sub_node));
     }
     gnc_price_commit_edit (p);
     return TRUE;
