@@ -44,6 +44,7 @@
 #endif
 #include <zlib.h>
 #include <errno.h>
+#include <cstdint>
 
 #include "gnc-engine.h"
 #include "gnc-pricedb-p.h"
@@ -51,7 +52,7 @@
 #include "SX-book.h"
 #include "SX-book-p.h"
 #include "Transaction.h"
-#include "TransactionP.h"
+#include "TransactionP.hpp"
 #include "TransLog.h"
 #if PLATFORM(WINDOWS)
 #ifdef __STRICT_ANSI_UNSET__
@@ -73,14 +74,6 @@
 #include "io-gncxml-v2.h"
 #include "io-gncxml-gen.h"
 
-/* Do not treat -Wstrict-aliasing warnings as errors because of problems of the
- * G_LOCK* macros as declared by glib.  See
- * https://bugs.gnucash.org/show_bug.cgi?id=316221 for additional information.
- */
-#if (__GNUC__ >= 4 && __GNUC_MINOR__ >= 2)
-#    pragma GCC diagnostic warning "-Wstrict-aliasing"
-#endif
-
 static QofLogModule log_module = GNC_MOD_IO;
 
 typedef struct
@@ -376,7 +369,6 @@ gnc_counter_end_handler (gpointer data_for_children,
                          gpointer parent_data, gpointer global_data,
                          gpointer* result, const gchar* tag)
 {
-    char* strval;
     gint64 val;
     char* type;
     xmlNodePtr tree = (xmlNodePtr)data_for_children;
@@ -399,11 +391,11 @@ gnc_counter_end_handler (gpointer data_for_children,
      * This is invalid xml because the namespace isn't declared in the
      * tag itself. This should be changed to 'type' at some point. */
     type = (char*)xmlGetProp (tree, BAD_CAST "cd:type");
-    strval = dom_tree_to_text (tree);
-    if (!string_to_gint64 (strval, &val))
+    if (!apply_xmlnode_text<bool> ([&val](auto txt){ return string_to_gint64 (txt, &val);}, tree))
     {
+        auto strval = dom_tree_to_text (tree);
         PERR ("string_to_gint64 failed with input: %s",
-              strval ? strval : "(null)");
+              strval ? strval->c_str() : "(null)");
         ret = FALSE;
     }
     else if (g_strcmp0 (type, "transaction") == 0)
@@ -456,7 +448,6 @@ gnc_counter_end_handler (gpointer data_for_children,
         }
     }
 
-    g_free (strval);
     xmlFree (type);
     xmlFreeNode (tree);
     return ret;
@@ -880,11 +871,11 @@ static gboolean
 write_counts (FILE* out, ...)
 {
     va_list ap;
-    char* type;
+    const char* type;
     gboolean success = TRUE;
 
     va_start (ap, out);
-    type = g_strdup (va_arg (ap, char*));
+    type = va_arg (ap, char*);
 
     while (success && type)
     {
@@ -893,6 +884,7 @@ write_counts (FILE* out, ...)
         if (amount != 0)
         {
 #if GNUCASH_REALLY_BUILD_AN_XML_TREE_ON_OUTPUT
+            char *type_dup = g_strdup (type);
             char* val;
             xmlNodePtr node;
 
@@ -904,10 +896,10 @@ write_counts (FILE* out, ...)
              * This is invalid xml because the namespace isn't
              * declared in the tag itself. This should be changed to
              * 'type' at some point. */
-            xmlSetProp (node, BAD_CAST "cd:type", checked_char_cast (type));
+            xmlSetProp (node, BAD_CAST "cd:type", checked_char_cast (type_dup));
             xmlNodeAddContent (node, checked_char_cast (val));
             g_free (val);
-            g_free (type);
+            g_free (type_dup);
 
             xmlElemDump (out, NULL, node);
             xmlFreeNode (node);
@@ -1605,19 +1597,24 @@ gnc_book_write_to_xml_file_v2 (QofBook* book, const char* filename,
 {
     bool success = true;
 
-    auto [file, thread] = try_gz_open (filename, "w", compress, TRUE);
+    auto [file, thread] = try_gz_open (filename, "wb", compress, TRUE);
     if (!file)
         return false;
 
     /* Try to write as much as possible */
-    success = (gnc_book_write_to_xml_filehandle_v2 (book, file));
+    if (!gnc_book_write_to_xml_filehandle_v2 (book, file))
+        success = false;
 
     /* Close the output stream */
-    success = ! (fclose (file));
+    if (fclose (file))
+        success = false;
 
     /* Optionally wait for parallel compression threads */
     if (thread)
-        success =  g_thread_join (thread) != nullptr;
+    {
+        if (g_thread_join (thread) == nullptr)
+            success = false;
+    }
 
     return success;
 }
@@ -1634,7 +1631,7 @@ gnc_book_write_accounts_to_xml_file_v2 (QofBackend* qof_be, QofBook* book,
     FILE* out;
     gboolean success = TRUE;
 
-    out = g_fopen (filename, "w");
+    out = g_fopen (filename, "wb");
 
     /* Try to write as much as possible */
     if (!out
