@@ -54,13 +54,12 @@ test_string_converters (void)
     {
         const char* mark = test_strings[i];
         xmlNodePtr test_node = text_to_dom_tree ("test-string", mark);
-        char* backout = dom_tree_to_text (test_node);
+        auto backout = dom_tree_to_text (test_node);
 
         do_test_args (
-            g_strcmp0 (backout, mark) == 0,
+            g_strcmp0 (backout->c_str(), mark) == 0,
             "string converting", __FILE__, __LINE__, "with string %s", mark);
 
-        g_free (backout);
         xmlFreeNode (test_node);
     }
 }
@@ -72,12 +71,11 @@ test_bad_string (void)
     const char* sanitized = "foo?bar";
     xmlNodePtr test_node = text_to_dom_tree ("test-string", badstr);
 
-    char* backout = dom_tree_to_text (test_node);
-    do_test_args (g_strcmp0 (backout, sanitized) == 0,
+    auto backout = dom_tree_to_text (test_node);
+    do_test_args (g_strcmp0 (backout->c_str(), sanitized) == 0,
                   "string sanitizing", __FILE__, __LINE__,
                   "with string %s", badstr);
 
-    g_free (backout);
     xmlFreeNode (test_node);
 }
 
