@@ -40,9 +40,9 @@
 #include <test-engine-stuff.h>
 #include <unittest-support.h>
 
-#include <AccountP.h>
+#include <AccountP.hpp>
 #include <Transaction.h>
-#include <TransactionP.h>
+#include <TransactionP.hpp>
 
 #include "../gnc-xml-helper.h"
 #include "../gnc-xml.h"
@@ -70,25 +70,22 @@ find_appropriate_node (xmlNodePtr node, Split* spl)
         {
             if (g_strcmp0 ((char*)mark2->name, "split:value") == 0)
             {
-                gnc_numeric* num = dom_tree_to_gnc_numeric (mark2);
+                gnc_numeric num = dom_tree_to_gnc_numeric (mark2);
 
-                if (gnc_numeric_equal (*num, xaccSplitGetValue (spl)))
+                if (gnc_numeric_equal (num, xaccSplitGetValue (spl)))
                 {
                     amount_good = TRUE;
                 }
-
-                g_free (num);
             }
             else if (g_strcmp0 ((char*)mark2->name, "split:account") == 0)
             {
-                GncGUID* accid = dom_tree_to_guid (mark2);
+                auto accid = dom_tree_to_guid (mark2);
                 Account* account = xaccSplitGetAccount (spl);
 
-                if (guid_equal (accid, xaccAccountGetGUID (account)))
+                if (guid_equal (&*accid, xaccAccountGetGUID (account)))
                 {
                     account_guid_good = TRUE;
                 }
-                guid_free (accid);
             }
 
             if (account_guid_good && amount_good)
@@ -110,89 +107,77 @@ equals_node_val_vs_split_internal (xmlNodePtr node, Split* spl)
     {
         if (g_strcmp0 ((char*)mark->name, "split:id") == 0)
         {
-            GncGUID* id = dom_tree_to_guid (mark);
+            auto id = dom_tree_to_guid (mark);
 
-            if (!guid_equal (id, xaccSplitGetGUID (spl)))
+            if (!guid_equal (&*id, xaccSplitGetGUID (spl)))
             {
-                guid_free (id);
                 return "ids differ";
             }
-            guid_free (id);
         }
         else if (g_strcmp0 ((char*)mark->name, "split:memo") == 0)
         {
-            char* memo = dom_tree_to_text (mark);
+            auto memo = dom_tree_to_text (mark);
 
-            if (g_strcmp0 (memo, xaccSplitGetMemo (spl)) != 0)
+            if (g_strcmp0 (memo->c_str(), xaccSplitGetMemo (spl)) != 0)
             {
-                g_free (memo);
                 return "memos differ";
             }
-            g_free (memo);
         }
         else if (g_strcmp0 ((char*)mark->name, "split:reconciled-state") == 0)
         {
-            char* rs = dom_tree_to_text (mark);
+            auto rs = dom_tree_to_text (mark);
 
-            if (rs[0] != xaccSplitGetReconcile (spl))
+            if (!rs || rs->front() != xaccSplitGetReconcile (spl))
             {
-                g_free (rs);
                 return "states differ";
             }
-            g_free (rs);
         }
         else if (g_strcmp0 ((char*)mark->name, "split:value") == 0)
         {
-            gnc_numeric* num = dom_tree_to_gnc_numeric (mark);
+            gnc_numeric num = dom_tree_to_gnc_numeric (mark);
             gnc_numeric val = xaccSplitGetValue (spl);
 
-            if (!gnc_numeric_equal (*num, val))
+            if (!gnc_numeric_equal (num, val))
             {
-                g_free (num);
                 return g_strdup_printf ("values differ: %" G_GINT64_FORMAT "/%"
                                         G_GINT64_FORMAT " v %" G_GINT64_FORMAT
                                         "/%" G_GINT64_FORMAT,
-                                        (*num).num, (*num).denom,
+                                        num.num, num.denom,
                                         val.num, val.denom);
             }
-            g_free (num);
         }
         else if (g_strcmp0 ((char*)mark->name, "split:quantity") == 0)
         {
-            gnc_numeric* num = dom_tree_to_gnc_numeric (mark);
+            gnc_numeric num = dom_tree_to_gnc_numeric (mark);
             gnc_numeric val = xaccSplitGetAmount (spl);
 
-            if (!gnc_numeric_equal (*num, val))
+            if (!gnc_numeric_equal (num, val))
             {
                 return g_strdup_printf ("quantities differ under _equal: %"
                                         G_GINT64_FORMAT "/%" G_GINT64_FORMAT
                                         " v %" G_GINT64_FORMAT "/%"
                                         G_GINT64_FORMAT,
-                                        (*num).num, (*num).denom,
+                                        num.num, num.denom,
                                         val.num, val.denom);
             }
-            if (!gnc_numeric_equal (*num, val))
+            if (!gnc_numeric_equal (num, val))
             {
-                g_free (num);
                 return g_strdup_printf ("quantities differ: %" G_GINT64_FORMAT
                                         "/%" G_GINT64_FORMAT " v %"
                                         G_GINT64_FORMAT "/%" G_GINT64_FORMAT,
-                                        (*num).num, (*num).denom,
+                                        num.num, num.denom,
                                         val.num, val.denom);
             }
-            g_free (num);
         }
         else if (g_strcmp0 ((char*)mark->name, "split:account") == 0)
         {
-            GncGUID* id = dom_tree_to_guid (mark);
+            auto id = dom_tree_to_guid (mark);
             Account* account = xaccSplitGetAccount (spl);
 
-            if (!guid_equal (id, xaccAccountGetGUID (account)))
+            if (!guid_equal (&*id, xaccAccountGetGUID (account)))
             {
-                guid_free (id);
                 return "accounts differ";
             }
-            guid_free (id);
         }
     }
     return NULL;
