@@ -42,19 +42,19 @@ usr_confpath_strings strs2[] =
 {
     {
         0, "gnc_build_userdata_path",
-        PACKAGE
+        PROJECT_NAME
     },
     {
         1, "gnc_build_book_path",
-        PACKAGE G_DIR_SEPARATOR_S "books"
+        PROJECT_NAME G_DIR_SEPARATOR_S "books"
     },
     {
         2, "gnc_build_translog_path",
-        PACKAGE G_DIR_SEPARATOR_S "translog"
+        PROJECT_NAME G_DIR_SEPARATOR_S "translog"
     },
     {
         3, "gnc_build_data_path",
-        PACKAGE G_DIR_SEPARATOR_S "data"
+        PROJECT_NAME G_DIR_SEPARATOR_S "data"
     },
     { 0, NULL, NULL },
 };
@@ -72,6 +72,7 @@ main(G_GNUC_UNUSED int argc, G_GNUC_UNUSED char **argv)
     const char *tmp_dir = g_get_tmp_dir();
     const char *builddir = g_getenv ("GNC_BUILDDIR");
     char *homedir = g_build_filename (builddir, "notexist", NULL);
+    char *testdatahome = g_build_filename (homedir, "gnucash", NULL);
 
     /* Assume we're not in a build environment to test
      * the function's actual behaviour in a real world use case, using
@@ -83,6 +84,11 @@ main(G_GNUC_UNUSED int argc, G_GNUC_UNUSED char **argv)
      * The code should fall back to using the temporary
      * directory in that case. */
     g_setenv("HOME", homedir, TRUE);
+    g_setenv("GNC_DATA_HOME", testdatahome, TRUE);
+    g_setenv("XDG_DATA_HOME", "", TRUE);
+    g_free (homedir);
+    g_free (testdatahome);
+
     for (i = 0; strs2[i].funcname != NULL; i++)
     {
         char *daout;
