@@ -210,6 +210,10 @@ typedef enum
      *    (by calling xaccAccountBeginEdit()) before calling this routine.*/
     void xaccAccountDestroy (Account *account);
 
+    /** Destroy all of the transactions that parent splits in an account.
+     */
+    void xaccAccountDestroyAllTransactions(Account *acc);
+
     /** Compare two accounts for equality - this is a deep compare. */
     gboolean xaccAccountEqual(const Account *a, const Account* b,
                               gboolean check_guids);
@@ -312,6 +316,11 @@ typedef enum
     void xaccAccountSetSortReversed (Account *account, gboolean sortreversed);
     /** Set the account's notes */
     void xaccAccountSetNotes (Account *account, const char *notes);
+
+    /** Set the account's associated account e.g. stock account -> dividend account */
+    void xaccAccountSetAssociatedAccount (Account *acc, const char *tag,
+                                          const Account *assoc_acct);
+
     /** Set the last num field of an Account */
     void xaccAccountSetLastNum (Account *account, const char *num);
     /** Set the account's lot order policy */
@@ -416,6 +425,9 @@ typedef enum
     gboolean xaccAccountGetSortReversed (const Account *account);
     /** Get the account's notes */
     const char * xaccAccountGetNotes (const Account *account);
+
+    /** Get the account's associated account e.g. stock account -> dividend account */
+    Account* xaccAccountGetAssociatedAccount (const Account *acc, const char *tag);
     /** Get the last num field of an Account */
     const char * xaccAccountGetLastNum (const Account *account);
     /** Get the account's lot order policy */
@@ -618,6 +630,8 @@ typedef enum
 
     gnc_numeric xaccAccountGetNoclosingBalanceChangeForPeriod (
         Account *acc, time64 date1, time64 date2, gboolean recurse);
+    gnc_numeric xaccAccountGetNoclosingBalanceChangeInCurrencyForPeriod (
+        Account *acc, time64 date1, time64 date2, gboolean recurse);          
     gnc_numeric xaccAccountGetBalanceChangeForPeriod (
         Account *acc, time64 date1, time64 date2, gboolean recurse);
 
@@ -1050,6 +1064,8 @@ typedef enum
      */
     SplitList* xaccAccountGetSplitList (const Account *account);
 
+    size_t xaccAccountGetSplitsSize (const Account *account);
+
     /** The xaccAccountMoveAllSplits() routine reassigns each of the splits
      *  in accfrom to accto. */
     void xaccAccountMoveAllSplits (Account *accfrom, Account *accto);
@@ -1394,9 +1410,9 @@ typedef enum
     const char * xaccAccountGetTaxUSPayerNameSource (const Account *account);
     /** DOCUMENT ME! */
     void xaccAccountSetTaxUSPayerNameSource (Account *account, const char *source);
-    /** DOCUMENT ME! */
+    /** Returns copy_number stored in KVP; if KVP doesn't exist or copy_number is zero, returns 1 */
     gint64 xaccAccountGetTaxUSCopyNumber (const Account *account);
-    /** DOCUMENT ME! */
+    /** Saves copy_number in KVP if it is greater than 1; if copy_number is zero, deletes KVP */
     void xaccAccountSetTaxUSCopyNumber (Account *account, gint64 copy_number);
     /** @} */
 
@@ -1481,12 +1497,6 @@ typedef enum
      */
     void gnc_account_tree_begin_staged_transaction_traversals(Account *acc);
 
-    /** xaccSplitsBeginStagedTransactionTraversals() resets the traversal
-     *    marker for each transaction which is a parent of one of the
-     *    splits in the list.
-     */
-    void xaccSplitsBeginStagedTransactionTraversals(SplitList *splits);
-
     /** xaccAccountBeginStagedTransactionTraversals() resets the traversal
      *    marker for each transaction which is a parent of one of the
      *    splits in the account.
@@ -1609,6 +1619,11 @@ typedef enum
         char           *count;
     }GncImapInfo;
 
+    /** Clean destructor for the imap_info structure of Bayesian
+     *  mappings
+     */
+    void gnc_account_imap_info_destroy (GncImapInfo*);
+
     /** Returns a GList of structure imap_info of all Bayesian mappings for
      *  required Account
      */
@@ -1676,8 +1691,6 @@ typedef enum
     const char * dxaccAccountGetQuoteTZ (const Account *account);
     /** @} */
 
-    GList * gnc_accounts_and_all_descendants (GList *accounts);
-
     /** @name Account parameter names
      @{
     */
