@@ -37,11 +37,11 @@
 struct _gncTaxTable
 {
     QofInstance     inst;
-    char *          name;
+    const char *    name;
     GncTaxTableEntryList*  entries;
     time64          modtime;      /* internal date of last modtime */
 
-    /* See src/doc/business.txt for an explanation of the following */
+    /* See libgnucash/engine/TaxTableBillTermImmutability.txt for an explanation of the following */
     /* Code that handles this is *identical* to that in gncBillTerm */
     gint64          refcount;
     GncTaxTable *   parent;       /* if non-null, we are an immutable child */
@@ -136,13 +136,9 @@ gncTaxIncludedStringToType (const char *str, GncTaxIncluded *type)
 #define _GNC_MOD_NAME        GNC_ID_TAXTABLE
 
 #define SET_STR(obj, member, str) { \
-        char * tmp; \
-        \
         if (!g_strcmp0 (member, str)) return; \
         gncTaxTableBeginEdit (obj); \
-        tmp = CACHE_INSERT (str); \
-        CACHE_REMOVE (member); \
-        member = tmp; \
+        CACHE_REPLACE (member, str); \
         }
 
 static inline void
@@ -216,7 +212,7 @@ enum
 };
 
 /* GObject Initialization */
-G_DEFINE_TYPE(GncTaxTable, gnc_taxtable, QOF_TYPE_INSTANCE);
+G_DEFINE_TYPE(GncTaxTable, gnc_taxtable, QOF_TYPE_INSTANCE)
 
 static void
 gnc_taxtable_init(GncTaxTable* tt)
@@ -232,7 +228,7 @@ gnc_taxtable_dispose(GObject *ttp)
 static void
 gnc_taxtable_finalize(GObject* ttp)
 {
-    G_OBJECT_CLASS(gnc_taxtable_parent_class)->dispose(ttp);
+    G_OBJECT_CLASS(gnc_taxtable_parent_class)->finalize(ttp);
 }
 
 static void
@@ -708,7 +704,7 @@ GncTaxTableList * gncTaxTableGetTables (QofBook *book)
     if (!book) return NULL;
 
     bi = qof_book_get_data (book, _GNC_MOD_NAME);
-    return bi->tables;
+    return bi ? bi->tables : NULL;
 }
 
 const char *gncTaxTableGetName (const GncTaxTable *table)
@@ -928,7 +924,7 @@ gboolean gncTaxTableEqual(const GncTaxTable *a, const GncTaxTable *b)
     }
 
 #if 0
-    /* See src/doc/business.txt for an explanation of the following */
+    /* See libgnucash/engine/TaxTableBillTermImmutability.txt for an explanation of the following */
     /* Code that handles this is *identical* to that in gncBillTerm */
     gint64          refcount;
     GncTaxTable *   parent;       /* if non-null, we are an immutable child */
@@ -1019,14 +1015,27 @@ static void _gncTaxTableCreate (QofBook *book)
     qof_book_set_data (book, _GNC_MOD_NAME, bi);
 }
 
+static void
+destroy_taxtable_on_book_close (QofInstance *ent, gpointer data)
+{
+    GncTaxTable *table = GNC_TAXTABLE(ent);
+
+    gncTaxTableBeginEdit (table);
+    gncTaxTableDestroy (table);
+}
+
 static void _gncTaxTableDestroy (QofBook *book)
 {
     struct _book_info *bi;
+    QofCollection *col;
 
     if (!book) return;
 
     bi = qof_book_get_data (book, _GNC_MOD_NAME);
 
+    col = qof_book_get_collection (book, GNC_ID_TAXTABLE);
+    qof_collection_foreach (col, destroy_taxtable_on_book_close, NULL);
+
     g_list_free (bi->tables);
     g_free (bi);
 }
