@@ -25,18 +25,14 @@
  *   Author: Linas Vepstas (linas@linas.org)                        *
  *   Author: Phil Longstaff (phil.longstaff@yahoo.ca)               *
 \********************************************************************/
+#include <glib.h>
 
-extern "C"
-{
 #include <config.h>
 
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include "qof.h"
-}
-
-#include <glib.h>
 
 /* Uncomment if you need to log anything.
 static QofLogModule log_module = QOF_MOD_UTIL;
@@ -85,7 +81,7 @@ qof_string_cache_destroy (void)
 void
 qof_string_cache_remove(const char * key)
 {
-    if (key)
+    if (key && key[0] != 0)
     {
         GHashTable* cache = qof_get_string_cache();
         gpointer value;
@@ -107,11 +103,16 @@ qof_string_cache_remove(const char * key)
 
 /* If the key exists in the cache, increment the refcount.  Otherwise,
  * add it with a refcount of 1. */
-char *
+const char *
 qof_string_cache_insert(const char * key)
 {
     if (key)
     {
+        if (key[0] == 0)
+        {
+            return "";
+        }
+
         GHashTable* cache = qof_get_string_cache();
         gpointer value;
         gpointer cache_key;
@@ -133,10 +134,10 @@ qof_string_cache_insert(const char * key)
     return NULL;
 }
 
-char *
+const char *
 qof_string_cache_replace(char const * dst, char const * src)
 {
-    char * tmp {qof_string_cache_insert (src)};
+    const char * tmp {qof_string_cache_insert (src)};
     qof_string_cache_remove (dst);
     return tmp;
 }
