@@ -24,9 +24,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *  02110-1301,  USA
  */
+#include <glib.h>
+#include <glib/gstdio.h>
 
-extern "C"
-{
 #include <config.h>
 
 #include <platform.h>
@@ -46,15 +46,12 @@ extern "C"
 #include <string.h>
 #include <stdio.h>
 
-}
-
-#include <glib.h>
-#include <glib/gstdio.h>
 #undef G_LOG_DOMAIN
 #define G_LOG_DOMAIN "qof.log"
 #include "qof.h"
 #include "qoflog.h"
 #include <string>
+#include <string_view>
 #include <vector>
 #include <memory>
 #include <algorithm>
@@ -64,11 +61,11 @@ extern "C"
 #define QOF_LOG_INDENT_WIDTH 4
 #define NUM_CLOCKS 10
 
-static FILE *fout = NULL;
-static gchar* function_buffer = NULL;
+static FILE *fout = nullptr;
+static gchar* function_buffer = nullptr;
 static gint qof_log_num_spaces = 0;
-static GLogFunc previous_handler = NULL;
-static gchar* qof_logger_format = NULL;
+static GLogFunc previous_handler = nullptr;
+static gchar* qof_logger_format = nullptr;
 static QofLogModule log_module = "qof";
 
 using StrVec = std::vector<std::string>;
@@ -79,9 +76,11 @@ using MEVec = std::vector<ModuleEntryPtr>;
 
 static constexpr int parts = 4; //Log domain parts vector preallocation size
 static constexpr QofLogLevel default_level = QOF_LOG_WARNING;
+static QofLogLevel current_max{default_level};
+
 struct ModuleEntry
 {
-    ModuleEntry(std::string name, QofLogLevel level) :
+    ModuleEntry(const std::string& name, QofLogLevel level) :
         m_name{name}, m_level{level} {
             m_children.reserve(parts);
         }
@@ -91,7 +90,7 @@ struct ModuleEntry
     MEVec m_children;
 };
 
-static ModuleEntryPtr _modules = NULL;
+static ModuleEntryPtr _modules = nullptr;
 
 static ModuleEntry*
 get_modules()
@@ -102,19 +101,19 @@ get_modules()
 }
 
 static StrVec
-split_domain (const std::string domain)
+split_domain (const std::string_view domain)
 {
     StrVec domain_parts;
     domain_parts.reserve(parts);
     int start = 0;
     auto pos = domain.find(".");
-    if (pos == std::string::npos)
+    if (pos == std::string_view::npos)
     {
         domain_parts.emplace_back(domain);
     }
     else
     {
-        while (pos != std::string::npos)
+        while (pos != std::string_view::npos)
         {
             auto part_name{domain.substr(start, pos - start)};
             domain_parts.emplace_back(part_name);
@@ -156,7 +155,7 @@ qof_log_set_file(FILE *outfile)
 void
 qof_log_init(void)
 {
-    qof_log_init_filename(NULL);
+    qof_log_init_filename(nullptr);
 }
 
 static void
@@ -181,14 +180,14 @@ log4glib_handler(const gchar     *log_domain,
 #endif
             ;
         const char *level_str = qof_log_level_to_string(level);
-        now = gnc_time (NULL);
+        now = gnc_time (nullptr);
         gnc_localtime_r (&now, &now_tm);
         qof_strftime(timestamp_buf, 9, format_24hour, &now_tm);
 
         fprintf(fout, qof_logger_format,
                 timestamp_buf,
                 5, level_str,
-                (log_domain == NULL ? "" : log_domain),
+                (log_domain == nullptr ? "" : log_domain),
                 qof_log_num_spaces, "",
                 message,
                 (g_str_has_suffix(message, "\n") ? "" : "\n"));
@@ -199,7 +198,7 @@ log4glib_handler(const gchar     *log_domain,
     else
     {
          // chain
-         previous_handler(log_domain, log_level, message, NULL);
+         previous_handler(log_domain, log_level, message, nullptr);
     }
     */
 }
@@ -218,7 +217,7 @@ qof_log_init_filename(const gchar* log_filename)
         int fd;
         gchar *fname;
 
-        if (fout != NULL && fout != stderr && fout != stdout)
+        if (fout != nullptr && fout != stderr && fout != stdout)
             fclose(fout);
 
         fname = g_strconcat(log_filename, ".XXXXXX.log", nullptr);
@@ -234,7 +233,7 @@ qof_log_init_filename(const gchar* log_filename)
             g_assert(g_strcmp0(log_filename, "/dev/null") != 0);
 
             /* Windows prevents renaming of open files, so the next command silently fails there
-             * No problem, the filename on Winows will simply have the random characters */
+             * No problem, the filename on Windows will simply have the random characters */
             g_rename(fname, log_filename);
             fout = fdopen(fd, "w");
 #endif
@@ -252,7 +251,7 @@ qof_log_init_filename(const gchar* log_filename)
     if (!fout)
         fout = stderr;
 
-    if (previous_handler == NULL)
+    if (previous_handler == nullptr)
         previous_handler = g_log_set_default_handler(log4glib_handler, modules);
 
     if (warn_about_missing_permission)
@@ -267,33 +266,42 @@ qof_log_shutdown (void)
     if (fout && fout != stderr && fout != stdout)
     {
         fclose(fout);
-        fout = NULL;
+        fout = nullptr;
+    }
+
+    if (qof_logger_format)
+    {
+        g_free (qof_logger_format);
+        qof_logger_format = nullptr;
     }
 
     if (function_buffer)
     {
         g_free(function_buffer);
-        function_buffer = NULL;
+        function_buffer = nullptr;
     }
 
-    if (_modules != NULL)
+    if (_modules != nullptr)
     {
         _modules = nullptr;
     }
 
-    if (previous_handler != NULL)
+    if (previous_handler != nullptr)
     {
-        g_log_set_default_handler(previous_handler, NULL);
-        previous_handler = NULL;
+        g_log_set_default_handler(previous_handler, nullptr);
+        previous_handler = nullptr;
     }
 }
 
 void
 qof_log_set_level(QofLogModule log_module, QofLogLevel level)
 {
-    if (!log_module || level == 0)
+    if (!log_module || level == QOF_LOG_FATAL)
         return;
 
+    if (level > current_max)
+        current_max = level;
+
     auto module_parts = split_domain(log_module);
     auto module = get_modules();
     for (auto part : module_parts)
@@ -321,10 +329,14 @@ qof_log_set_level(QofLogModule log_module, QofLogLevel level)
 gboolean
 qof_log_check(QofLogModule domain, QofLogLevel level)
 {
-
+// Check the global levels
+    if (level > current_max)
+        return FALSE;
+    if (level <= default_level)
+        return TRUE;
     auto module = get_modules();
-    // If the level is < the default then no need to look further.
-    if (level < module->m_level)
+    // If the level <= the default then no need to look further.
+    if (level <= module->m_level)
         return TRUE;
 
     if (!domain)
@@ -332,7 +344,7 @@ qof_log_check(QofLogModule domain, QofLogLevel level)
 
     auto domain_vec = split_domain(domain);
 
-    for (auto part : domain_vec)
+    for (const auto& part : domain_vec)
     {
         auto iter = std::find_if(module->m_children.begin(),
                                module->m_children.end(),
@@ -370,11 +382,11 @@ qof_log_prettify (const char *name)
     p = g_strstr_len (buffer, length, "(");
     if (p) *p = '\0';
     begin = g_strrstr (buffer, "*");
-    if (begin == NULL)
+    if (begin == nullptr)
         begin = g_strrstr (buffer, " ");
     else if (* (begin + 1) == ' ')
         ++ begin;
-    if (begin != NULL)
+    if (begin != nullptr)
         p = begin + 1;
     else
         p = buffer;
@@ -409,7 +421,7 @@ void
 qof_log_parse_log_config(const char *filename)
 {
     const gchar *levels_group = "levels", *output_group = "output";
-    GError *err = NULL;
+    GError *err = nullptr;
     GKeyFile *conf = g_key_file_new();
 
     if (!g_key_file_load_from_file(conf, filename, G_KEY_FILE_NONE, &err))
@@ -419,28 +431,28 @@ qof_log_parse_log_config(const char *filename)
         return;
     }
 
-    g_debug("parsing log config from [%s]", filename);
+    DEBUG("parsing log config from [%s]", filename);
     if (g_key_file_has_group(conf, levels_group))
     {
         gsize num_levels;
         unsigned int key_idx;
         gchar **levels;
         gint logger_max_name_length = 12;
-        gchar *str = NULL;
+        gchar *str = nullptr;
 
-        levels = g_key_file_get_keys(conf, levels_group, &num_levels, NULL);
+        levels = g_key_file_get_keys(conf, levels_group, &num_levels, nullptr);
 
-        for (key_idx = 0; key_idx < num_levels && levels[key_idx] != NULL; key_idx++)
+        for (key_idx = 0; key_idx < num_levels && levels[key_idx] != nullptr; key_idx++)
         {
             QofLogLevel level;
-            gchar *logger_name = NULL, *level_str = NULL;
+            gchar *logger_name = nullptr, *level_str = nullptr;
 
             logger_name = g_strdup(levels[key_idx]);
             logger_max_name_length = MAX (logger_max_name_length, (gint) strlen (logger_name));
-            level_str = g_key_file_get_string(conf, levels_group, logger_name, NULL);
+            level_str = g_key_file_get_string(conf, levels_group, logger_name, nullptr);
             level = qof_log_level_from_string(level_str);
 
-            g_debug("setting log [%s] to level [%s=%d]", logger_name, level_str, level);
+            DEBUG("setting log [%s] to level [%s=%d]", logger_name, level_str, level);
             qof_log_set_level(logger_name, level);
 
             g_free(logger_name);
@@ -462,8 +474,8 @@ qof_log_parse_log_config(const char *filename)
         unsigned int output_idx;
         gchar **outputs;
 
-        outputs = g_key_file_get_keys(conf, output_group, &num_outputs, NULL);
-        for (output_idx = 0; output_idx < num_outputs && outputs[output_idx] != NULL; output_idx++)
+        outputs = g_key_file_get_keys(conf, output_group, &num_outputs, nullptr);
+        for (output_idx = 0; output_idx < num_outputs && outputs[output_idx] != nullptr; output_idx++)
         {
             gchar *key = outputs[output_idx];
             gchar *value;
@@ -474,8 +486,8 @@ qof_log_parse_log_config(const char *filename)
                 continue;
             }
 
-            value = g_key_file_get_string(conf, output_group, key, NULL);
-            g_debug("setting [output].to=[%s]", value);
+            value = g_key_file_get_string(conf, output_group, key, nullptr);
+            DEBUG("setting [output].to=[%s]", value);
             qof_log_init_filename_special(value);
             g_free(value);
         }
@@ -485,14 +497,6 @@ qof_log_parse_log_config(const char *filename)
     g_key_file_free(conf);
 }
 
-void
-qof_log_set_default(QofLogLevel log_level)
-{
-    qof_log_set_level("", log_level);
-    qof_log_set_level("qof", log_level);
-    qof_log_set_level("qof.unknown", log_level);
-}
-
 const gchar*
 qof_log_level_to_string(QofLogLevel log_level)
 {
